;----------------------------------------------------------------------------------
;	FILE:			HVLLC-DPL-ISR.asm
;
;	Description:	HVLLC-DPL-ISR.asm contains the ISR for the system
;					It also contains the initailization routine for all the macros 
;					being used in the system both for CLA and C28x macros
;   
;   Revision/ Version: See HVLLC-Main.c
;----------------------------------------------------------------------------------

		;Gives peripheral addresses visibility in assembly
	    .cdecls   C,LIST,"PeripheralHeaderIncludes.h"

		;include C header file - sets INCR_BUILD etc.(used in conditional builds)
		.cdecls C,NOLIST, "IsoACLighting-Settings.h"

		;Include files for the Power Library Maco's being used by the system
		.include "ADCDRV_1ch.asm"
		.include "CNTL_2P2Z.asm"
		.include "PWMDRV_DualUpDwnCnt.asm"
		.include "PWMDRV_LLC_ComplPairDB.asm"
			
;=============================================================================
; Digital Power library - Initailization Routine 
;=============================================================================

		; label to DP initialisation function
		.def _DPL_Init	

		; dummy variable for pointer initialization
ZeroNet	.usect "ZeroNet_Section",2,1,1	; output terminal 1
tsPtr	.usect "ISR_data",2,1,1

		.text
_DPL_Init:
		ZAPA
		MOVL	XAR0, #ZeroNet
		MOVL	*XAR0, ACC

		; Initialize all the DP library macro used here 
		;---------------------------------------------------------
		.if(INCR_BUILD = 1)
			ADCDRV_1ch_INIT 1	; Vout 
			ADCDRV_1ch_INIT 2	; Ipri
			
			ADCDRV_1ch_INIT 8	; Iled1
			ADCDRV_1ch_INIT 9	; Iled2
			ADCDRV_1ch_INIT 10	; Iled3
			ADCDRV_1ch_INIT 11	; Iled4
			ADCDRV_1ch_INIT 12	; Iled5
			ADCDRV_1ch_INIT 13	; Iled6
			
			PWMDRV_LLC_ComplPairDB_INIT 4		; PWM1AB	
			
			PWMDRV_DualUpDwnCnt_INIT 1
			PWMDRV_DualUpDwnCnt_INIT 2
			PWMDRV_DualUpDwnCnt_INIT 3
			
		.endif
		;---------------------------------------------------------			
		.if(INCR_BUILD = 2)
			ADCDRV_1ch_INIT 1	; Vout
			ADCDRV_1ch_INIT 2	; Ipri
	
			ADCDRV_1ch_INIT 8	; Iled1
			ADCDRV_1ch_INIT 9	; Iled2
			ADCDRV_1ch_INIT 10	; Iled3
			ADCDRV_1ch_INIT 11	; Iled4
			ADCDRV_1ch_INIT 12	; Iled5
			ADCDRV_1ch_INIT 13	; Iled6

			CNTL_2P2Z_INIT 1
			CNTL_2P2Z_INIT 2
			CNTL_2P2Z_INIT 3
			CNTL_2P2Z_INIT 4
			CNTL_2P2Z_INIT 5
			CNTL_2P2Z_INIT 6
			CNTL_2P2Z_INIT 7
			
			PWMDRV_LLC_ComplPairDB_INIT 4		; PWM1AB	
			
			PWMDRV_DualUpDwnCnt_INIT 1
			PWMDRV_DualUpDwnCnt_INIT 2
			PWMDRV_DualUpDwnCnt_INIT 3
			
			MOVW	DP, #tsPtr												  ;
			MOV 	@tsPtr,#1	
		.endif
		;---------------------------------------------------------			
		LRETR

;-----------------------------------------------------------------------------------------

; Digital Power library - Interrupt Service Routine

		.sect "ramfuncs"
		; label to DP ISR Run function
		.def	_DPL_ISR

_DPL_ISR:
		; full context save - push any unprotected registers onto stack
		PUSH  	AR1H:AR0H
		PUSH  	XAR2
		PUSH  	XAR3
		PUSH  	XAR4
		PUSH  	XAR5
		PUSH  	XAR6
		PUSH  	XAR7
		PUSH  	XT
		SPM   	0          				; set C28 mode
		CLRC  	AMODE       
		CLRC  	PAGE0,OVM 
;		CLRC	INTM					; clear interrupt mask - comment if ISR non-nestable
;-----------------------------------------------------------------------------------------

; call DP library modules

		;---------------------------------------------------------
		.if(INCR_BUILD = 1)
			ADCDRV_1ch 1	; Vout
			ADCDRV_1ch 2	; Ipri
			ADCDRV_1ch 8	; Iled1
			ADCDRV_1ch 9	; Iled2
			ADCDRV_1ch 10	; Iled3
			ADCDRV_1ch 11	; Iled4
			ADCDRV_1ch 12	; Iled5
			ADCDRV_1ch 13	; Iled6

			;PWMDRV_LLC_ComplPairDB 4   ; LLC update done not done here; done in its own ISR
			
			PWMDRV_DualUpDwnCnt 1
			PWMDRV_DualUpDwnCnt 2
			PWMDRV_DualUpDwnCnt 3
		.endif
		;----------------------------------------------------------
		.if(INCR_BUILD = 2)
		; "14 to 18" cycles(from ISR trigger point to interrupt) + 11 cycles (context save)
		;   = 25 to 29 cycles to this point
			
		ADCDRV_1ch 1	; Vout
		CNTL_2P2Z 1
		;PWMDRV_LLC_ComplPairDB 4   ; LLC update done not done here; done in its own ISR
			
		MOVW	DP, #tsPtr
		CMP		@tsPtr, #1
		B		TS1, EQ
		CMP		@tsPtr, #3
		B		TS2, EQ
		CMP		@tsPtr, #5
		B		TS3, EQ
												  
		;MOV 	@tsPtr,#1
		LB		TS_SKIP
TS1:
		ADCDRV_1ch 8	; Iled0
		ADCDRV_1ch 9	; Iled1
		CNTL_2P2Z 2
		CNTL_2P2Z 3
		
		PWMDRV_DualUpDwnCnt 1	
		LB 		TS_SKIP
TS2:
		ADCDRV_1ch 10	; Iled2
		ADCDRV_1ch 11	; Iled3		
		CNTL_2P2Z 4
		CNTL_2P2Z 5
		
		PWMDRV_DualUpDwnCnt 2
		LB		TS_SKIP
TS3:
		ADCDRV_1ch 12	; Iled4
		ADCDRV_1ch 13	; Iled5			
		CNTL_2P2Z 6
		CNTL_2P2Z 7
		
		PWMDRV_DualUpDwnCnt 3
TS_SKIP:
		MOVW	DP, #tsPtr
		INC		@tsPtr
		CMP		@tsPtr, #6		; if new value of tsPtr is greater than 6, reset tsPtr to 1
		B		TS_FIN, LEQ		
		MOV		@tsPtr,	#1

TS_FIN:
			
		.endif
		;----------------------------------------------------------
		
;-----------------------------------------------------------------------------------------
; Interrupt management before exit

	.if(EPWMn_DPL_ISR=1)

	;.if(EPWM1)
		MOVW 	DP,#_EPwm1Regs.ETCLR
		MOV 	@_EPwm1Regs.ETCLR,#0x01			; Clear EPWM1 Int flag
	;.endif ; EPWM1
	
		MOVW 	DP,#_PieCtrlRegs.PIEACK			; Acknowledge PIE interrupt Group 3
		MOV 	@_PieCtrlRegs.PIEACK, #0x4
	.endif ; EPWMn_ISR

	.if(ADC_DPL_ISR=1)
	; Case where ISR is triggered by ADC 
;		MOVW 	DP,#_AdcRegs.ADCINTFLGCLR
;		MOV 	@AdcRegs.ADCINTFLGCLR, #0x01	; Clear ADCINT1 Flag

		MOVW 	DP,#_PieCtrlRegs.PIEACK			; Acknowledge PIE interrupt Group 1
		MOV 	@_PieCtrlRegs.PIEACK, #0x1
	.endif 

;-----------------------------------------------------------------------------------------
; full context restore
;		SETC	INTM					; set INTM to protect context restore
		POP   	XT
		POP   	XAR7
		POP   	XAR6
		POP   	XAR5
		POP   	XAR4
		POP   	XAR3
		POP   	XAR2
		POP   	AR1H:AR0H
		IRET							; return from interrupt
		
		
		; label to PWM ISR Run function
		.def	_RESPWM_ISR
_RESPWM_ISR:
		; full context save - push any unprotected registers onto stack
		PUSH  	AR1H:AR0H
		PUSH  	XAR2
		PUSH  	XAR3
		PUSH  	XAR4
		PUSH  	XAR5
		PUSH  	XAR6
		PUSH  	XAR7
		PUSH  	XT
		SPM   	0          				; set C28 mode
		CLRC  	AMODE       
		CLRC  	PAGE0,OVM 
		SETC	INTM					; set interrupt global mask - comment if ISR is nestable
;-----------------------------------------------------------------------------------------

; call DP library modules
		;---------------------------------------------------------
		.if(INCR_BUILD = 1)
			PWMDRV_LLC_ComplPairDB 4
		.endif
		;----------------------------------------------------------
		.if(INCR_BUILD = 2)
			PWMDRV_LLC_ComplPairDB 4
		.endif
		;----------------------------------------------------------
		
;-----------------------------------------------------------------------------------------
; Interrupt management before exit
		MOVW 	DP,#_EPwm4Regs.ETCLR
		MOV 	@_EPwm4Regs.ETCLR,#0x01			; Clear ePWM4 Int flag
	
		MOVW 	DP,#_PieCtrlRegs.PIEACK			
		MOV 	@_PieCtrlRegs.PIEACK, #0x4		; Acknowledge PIE interrupt Group 3

;-----------------------------------------------------------------------------------------
; full context restore
		CLRC	INTM					; clear interrupt global mask
		POP   	XT
		POP   	XAR7
		POP   	XAR6
		POP   	XAR5
		POP   	XAR4
		POP   	XAR3
		POP   	XAR2
		POP   	AR1H:AR0H
;-----------------------------------------------------------------------------------------
		IRET							; return from interrupt
		
		.end

; end of file

