//----------------------------------------------------------------------------------
//	FILE:			IsoACLighting-Main.C
//
//	Description:	Main file for the IsoACLighting kit project
//
//	Version: 		1.0
//
//  Target:  		TMS320F2802x(Piccolo)
//
//----------------------------------------------------------------------------------
//  Copyright Texas Instruments  2004-2012
//----------------------------------------------------------------------------------
//  Revision History:
//----------------------------------------------------------------------------------
//  Date	  | Description / Status
//----------------------------------------------------------------------------------
// January 07, 2011  - File created (BL)
// January 16, 2012  - Release (BL)
//----------------------------------------------------------------------------------
//
// PLEASE READ - Useful notes about this Project
//
// Although this project is made up of several files, the most important ones are:
//	 "IsoACLighting-Main.C"	- this file
//		- Application Initialization, Peripheral config,
//		- Application management
//		- Slower background code loops and Task scheduling
//	 "IsoACLighting-DevInit_F28xxx.C
//		- Device Initialization, e.g. Clock, PLL, WD, GPIO mapping
//		- Peripheral clock enables
//		- DevInit file will differ per each F28xxx device series, e.g. F280x, F2833x,
//	 "IsoACLighting-DPL-ISR.asm
//		- Assembly level library Macros and any cycle critical functions are found here
//	 "IsoACLighting-Settings.h"
//		- Global defines (settings) project selections are found here
//		- This file is referenced by both C and ASM files.
//
// Code is made up of sections, e.g. "FUNCTION PROTOTYPES", "VARIABLE DECLARATIONS" ,..etc
//	each section has FRAMEWORK and USER areas.
//  FRAMEWORK areas provide useful ready made "infrastructure" code which for the most part
//	does not need modification, e.g. Task scheduling, ISR call, GUI interface support,...etc
//  USER areas have functional example code which can be modified by USER to fit their appl.
//
// Code can be compiled with various build options (Incremental Builds IBx), these
//  options are selected in file "HVLLC-Settings.h".  Note: "Rebuild All" compile
//  tool bar button must be used if this file is modified.
//----------------------------------------------------------------------------------
#include "PeripheralHeaderIncludes.h"
#include "DSP2802x_EPWM_defines.h"		
#include "IsoACLighting-Settings.h"
		
#include "DPlib.h"	
#include "IQmathLib.h"
#include <math.h>

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// FUNCTION PROTOTYPES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Add protoypes of functions being used in the project here 
void DeviceInit(void);
#ifdef FLASH		
	void InitFlash();
#endif
extern void MemCopy(Uint16*, Uint16*, Uint16*);
//-------------------------------- DPLIB --------------------------------------------
void PWM_DualUpDwnCnt_CNF(int16 n, int16 Period, int16 mode, int16 phase);
void PWM_ComplPairDB_CNF(int16 n, int16 Period, int16 mode, int16 phase);
void PWM_ComplPairDB_UpdateDB(int16 n, int16 dbRED, int16 dbFED);
void ADC_SOC_CNF(int ChSel[], int TrigSel[], int ACQPS[], int IntChSel, int mode);

// -------------------------------- FRAMEWORK --------------------------------------
// State Machine function prototypes
//----------------------------------------------------------------------------------
// Alpha states
void A0(void);	//state A0
void B0(void);	//state B0
void C0(void);	//state C0

// A branch states
void A1(void);	//state A1
void A2(void);	//state A2

// B branch states
void B1(void);	//state B1

// C branch states
void C1(void);	//state C1
void C2(void);	//state C2

// Variable declarations
void (*Alpha_State_Ptr)(void);	// Base States pointer
void (*A_Task_Ptr)(void);		// State pointer A branch
void (*B_Task_Ptr)(void);		// State pointer B branch
void (*C_Task_Ptr)(void);		// State pointer C branch
//----------------------------------------------------------------------------------

void Comms_Init(void);
void Comms_Run(void);

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - GENERAL
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------
int16 VTimer0[4];	// Virtual Timers slaved off CPU Timer 0
int16 VTimer1[4];	// Virtual Timers slaved off CPU Timer 1
int16 VTimer2[4];	// Virtual Timers slaved off CPU Timer 2

int16 FlashID;

// Used for running BackGround in flash, and ISR in RAM
extern Uint16 RamfuncsLoadStart, RamfuncsLoadEnd, RamfuncsRunStart;

// Used for ADC Configuration 
int ChSel[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int TrigSel[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int ACQPS[16] = {8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8};
// ---------------------------------- USER -----------------------------------------
long PgainLLC;				// PID P gain
long IgainLLC;				// PID I gain
long DgainLLC;				// PID D gain

long PgainLED;				// PID P gain
long IgainLED;				// PID I gain
long DgainLED;				// PID D gain
// ---------------------------- DPLIB Net Pointers ---------------------------------
// Declare net pointers that are used to connect the DP Lib Macros  here 

extern interrupt void RESPWM_ISR(void);

// 2P2Z
extern volatile long *CNTL_2P2Z_Ref1;
extern volatile long *CNTL_2P2Z_Fdbk1;
extern volatile long *CNTL_2P2Z_Out1;
extern volatile long *CNTL_2P2Z_Coef1;

extern volatile long *CNTL_2P2Z_Ref2;
extern volatile long *CNTL_2P2Z_Fdbk2;
extern volatile long *CNTL_2P2Z_Out2;
extern volatile long *CNTL_2P2Z_Coef2;

extern volatile long *CNTL_2P2Z_Ref3;
extern volatile long *CNTL_2P2Z_Fdbk3;
extern volatile long *CNTL_2P2Z_Out3;
extern volatile long *CNTL_2P2Z_Coef3;

extern volatile long *CNTL_2P2Z_Ref4;
extern volatile long *CNTL_2P2Z_Fdbk4;
extern volatile long *CNTL_2P2Z_Out4;
extern volatile long *CNTL_2P2Z_Coef4;

extern volatile long *CNTL_2P2Z_Ref5;
extern volatile long *CNTL_2P2Z_Fdbk5;
extern volatile long *CNTL_2P2Z_Out5;
extern volatile long *CNTL_2P2Z_Coef5;

extern volatile long *CNTL_2P2Z_Ref6;
extern volatile long *CNTL_2P2Z_Fdbk6;
extern volatile long *CNTL_2P2Z_Out6;
extern volatile long *CNTL_2P2Z_Coef6;

extern volatile long *CNTL_2P2Z_Ref7;
extern volatile long *CNTL_2P2Z_Fdbk7;
extern volatile long *CNTL_2P2Z_Out7;
extern volatile long *CNTL_2P2Z_Coef7;

// ADC
extern volatile long *ADCDRV_1ch_Rlt0;	// instance #1, Vout (first sample)
extern volatile long *ADCDRV_1ch_Rlt1;	// instance #2, Vout
extern volatile long *ADCDRV_1ch_Rlt2;	// instance #3, Ipri

extern volatile long *ADCDRV_1ch_Rlt8;	// instance #8, V_SR2 
extern volatile long *ADCDRV_1ch_Rlt9;	// instance #9, V_SR2 
extern volatile long *ADCDRV_1ch_Rlt10;	// instance #10, V_SR2 
extern volatile long *ADCDRV_1ch_Rlt11;	// instance #11, V_SR2 
extern volatile long *ADCDRV_1ch_Rlt12;	// instance #12, V_SR2 
extern volatile long *ADCDRV_1ch_Rlt13;	// instance #13, V_SR2 

// PWM
extern volatile long *PWMDRV_LLC_ComplPairDB_Duty4;
extern volatile long *PWMDRV_LLC_ComplPairDB_Period4;
extern volatile long *PWMDRV_DualUpDwnCnt_Duty1A;
extern volatile long *PWMDRV_DualUpDwnCnt_Duty1B;
extern volatile long *PWMDRV_DualUpDwnCnt_Duty2A;
extern volatile long *PWMDRV_DualUpDwnCnt_Duty2B;
extern volatile long *PWMDRV_DualUpDwnCnt_Duty3A;
extern volatile long *PWMDRV_DualUpDwnCnt_Duty3B;

// ---------------------------- DPLIB Variables ---------------------------------
// Declare the net variables being used by the DP Lib Macro here 
volatile long Vout_LLC; 
volatile long Ipri_LLC; 
volatile long Iled[6];

volatile long llc_duty; 
volatile long llc_period;
volatile long led_duty[6];
long Zero;
long HighRef;

long Vset_LLC;				// Control loop target prior to slew rate limiting
long Vset_LLC_slewed;		// Control loop target after slew rate limiting

long Iset_LED[6];
long Iset_LED_slewed[6];

long SlewError;
long SlewStep_LLC;
long SlewStep_LED;

#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct_LLC, "CNTL_2P2Z_Coef");
#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct_LED, "CNTL_2P2Z_Coef");
struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct_LLC;
struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct_LED;

// Used to indirectly access all PWM modules
volatile struct EPWM_REGS *ePWM[] = 
 				  { &EPwm1Regs,			//intentional: (ePWM[0] not used)
				  	&EPwm1Regs,
					&EPwm2Regs,
					&EPwm3Regs,
					&EPwm4Regs,
					#if (!DSP2802x_DEVICE_H)
					&EPwm5Regs,
					&EPwm6Regs,
					#if (DSP2803x_DEVICE_H || DSP2806x_DEVICE_H)
					&EPwm7Regs,
					#if (DSP2806x_DEVICE_H)
					&EPwm8Regs
					#endif 
					#endif 
					#endif
				  };

// Used to indirectly access all Comparator modules
volatile struct COMP_REGS *Comp[] = 
 				  { &Comp1Regs,			//intentional: (Comp[0] not used)
					&Comp1Regs,				  
					&Comp2Regs,
					#if (DSP2803x_DEVICE_H || DSP2806x_DEVICE_H)
					&Comp3Regs
					#endif 
				  };

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - CCS WatchWindow / GUI support
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------

// ---------------------------------- USER -----------------------------------------

// Monitor ("Get")	// Display as:
Uint16 Gui_Vout_LLC;	// Q9
Uint16 Gui_Ipri_LLC;	// Q9	
Uint16 Gui_ILed[6]; // Q15

// Configure ("Set")
Uint16 Gui_Vset_LLC;	// Q9
Uint16 Gui_IsetLed[6];	// Q15
long LedMinCurrent;

// History arrays are used for Running Average calculation (boxcar filter)
// Used for CCS display and GUI only, not part of control loop processing
Uint16 Hist_Vout_LLC[HistorySize];
Uint16 Hist_Ipri_LLC[HistorySize];
Uint16 ILedH[6][HistorySize];	

//Scaling Constants (values found via spreadsheet; exact value calibrated per board)
Uint16 K_Vout_LLC;	// 
Uint16 K_Ipri_LLC;	// 
Uint16 iK_Vset_LLC;	//

int16 ResonantEnable;

Uint16 K_ILed;
Uint16 iK_ILed; 

int16 LEDs_linked;

// Variables for background support only (no need to access)
int16 i;						// common use incrementer
int16 j;						// common use incrementer

Uint32 HistPtr, temp_Scratch; 	// Temp here means Temporary


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE - starts here
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void main(void)
{
//=================================================================================
//	INITIALISATION - General
//=================================================================================

	// The DeviceInit() configures the clocks and pin mux registers 
	// The function is declared in HVLLC-DevInit_F2803/2x.c,
	// Please ensure/edit that all the desired components pin muxes 
	// are configured properly that clocks for the peripherals used
	// are enabled, for example the individual PWM clock must be enabled 
	// along with the Time Base Clock 

	DeviceInit();	// Device Life support & GPIO

//-------------------------------- FRAMEWORK --------------------------------------

// Only used if running from FLASH
// Note that the variable FLASH is defined by the compiler with -d FLASH

#ifdef FLASH		
// Copy time critical code and Flash setup code to RAM
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the linker files. 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
	InitFlash();	// Call the flash wrapper init function
#endif //(FLASH)


// Timing sync for background loops
// Timer period definitions found in PeripheralHeaderIncludes.h
	CpuTimer0Regs.PRD.all =  mSec0_5;	// A tasks
	CpuTimer1Regs.PRD.all =  mSec2;		// B tasks
	CpuTimer2Regs.PRD.all =  mSec10;	// C tasks

// Tasks State-machine init
	Alpha_State_Ptr = &A0;
	A_Task_Ptr = &A1;
	B_Task_Ptr = &B1;
	C_Task_Ptr = &C1;

	VTimer0[0] = 0;	
	VTimer1[0] = 0;
	VTimer2[0] = 0;
	
	FlashID = 0x2020;

// ---------------------------------- USER -----------------------------------------
//  put common initialization/variable definitions here

	PgainLLC=_IQ26(0.85004515882);				//
	IgainLLC=_IQ26(0.02250133108);				//
	DgainLLC=_IQ26(0.0000000300);				//

	PgainLED=_IQ26(0.25);				//
	IgainLED=_IQ26(0.045);				//
	DgainLED=_IQ26(0.0000000300);				//

	K_Vout_LLC=26487;	// 0.808 (Q15) 
	K_Ipri_LLC=20275;	// 0.619 (Q15) 
	iK_Vset_LLC=20269;	// 1.237 (Q14)
	
	K_ILed = 18022;		// 0.55 in Q15  (see excel spreadsheet)
	iK_ILed = 29789;	// 1.818 in Q14 (see excel spreadsheet)

	Vset_LLC_slewed = 0;
	Gui_Vset_LLC=0;
	Vset_LLC=0;
	
	LEDs_linked = 0;
	LedMinCurrent = 200000;  // 52255~= 0.0035% of max current
	
	for(i = 0; i < 6; i++)
	{
		Iset_LED[i] = 0;
		Gui_IsetLed[i] = 0;
		Iset_LED_slewed[i] = 0;
	}
	
	llc_duty=_IQ24(0.5);
	llc_period=_IQ14(550);
	
	led_duty[0] = _IQ24(0.00);
	led_duty[1] = _IQ24(0.00);
	led_duty[2] = _IQ24(0.00);
	led_duty[3] = _IQ24(0.00);
	led_duty[4] = _IQ24(0.00);
	led_duty[5] = _IQ24(0.00);
	
	ResonantEnable=0;
	
	CNTL_2P2Z_CoefStruct_LLC.b2 = LLC_CONTROL_POLARITY*(DgainLLC);
	CNTL_2P2Z_CoefStruct_LLC.b1 = LLC_CONTROL_POLARITY*(IgainLLC - PgainLLC - DgainLLC - DgainLLC);
	CNTL_2P2Z_CoefStruct_LLC.b0 = LLC_CONTROL_POLARITY*(PgainLLC + IgainLLC + DgainLLC);
	CNTL_2P2Z_CoefStruct_LLC.a2 = _IQ26(0.0);
	CNTL_2P2Z_CoefStruct_LLC.a1 = _IQ26(1.0);
	CNTL_2P2Z_CoefStruct_LLC.max =_IQ14(LLC_MAX_PERIOD);
	CNTL_2P2Z_CoefStruct_LLC.min =_IQ14(LLC_MIN_PERIOD);

	CNTL_2P2Z_CoefStruct_LED.b2 = DgainLED;
	CNTL_2P2Z_CoefStruct_LED.b1 = IgainLED - PgainLED - DgainLED - DgainLED;
	CNTL_2P2Z_CoefStruct_LED.b0 = PgainLED + IgainLED + DgainLED;
	CNTL_2P2Z_CoefStruct_LED.a2 = _IQ26(0.0);
	CNTL_2P2Z_CoefStruct_LED.a1 = _IQ26(1.0);
	CNTL_2P2Z_CoefStruct_LED.max =_IQ24(0.95);
	CNTL_2P2Z_CoefStruct_LED.min =_IQ24(0.0);

//=================================================================================
//	Configure ADC and PWM Peripherals
//=================================================================================

// ADC and high-level PWM configuration
#define LLC_PERIOD 500	//500 cycles -> 120KHz @60MHz CPU		(up count mode)
#define	LED_PERIOD 1920 //1920 cycles -> 31.25KHz @60MHz CPU	(up-down count mode; 960 cycles up, 960 cycles down)
#define LED_PERIOD_ONESIXTHS	320
#define LED_PERIOD_TWOSIXTHS	640

#define RED 0x0010		// 3 (cycles) -> 50ns delay
#define FED 0X0010		// 3 (cycles) -> 50ns delay

#define		VoutR	AdcResult.ADCRESULT1		//Q12
#define		IpriR	AdcResult.ADCRESULT2		//Q12
#define		LED1R	AdcResult.ADCRESULT8		//Q12
#define		LED2R	AdcResult.ADCRESULT9		//Q12
#define		LED3R	AdcResult.ADCRESULT10		//Q12
#define		LED4R	AdcResult.ADCRESULT11		//Q12
#define		LED5R	AdcResult.ADCRESULT12		//Q12
#define		LED6R	AdcResult.ADCRESULT13		//Q12
#define 	Vpfcout	AdcResult.ADCRESULT14 		//Q12

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
	EDIS;

	// Configure PWM4AB 
	PWM_ComplPairDB_CNF(4, LLC_PERIOD, 1, 0); 
	PWM_ComplPairDB_UpdateDB(4, RED, FED);
	
	PWM_DualUpDwnCnt_CNF(1,LED_PERIOD, 1, 0);
	PWM_DualUpDwnCnt_CNF(2,LED_PERIOD, 0, LED_PERIOD_ONESIXTHS);
	PWM_DualUpDwnCnt_CNF(3,LED_PERIOD, 0, LED_PERIOD_TWOSIXTHS);
	
	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;		//synchronize PWM modules
	EDIS;
	
	// ADC Channel Selection
	ChSel[0] = 2;		// 2  = A2 - Vout (to avoid first sample in pre-RevA Piccolo errata)
	ChSel[1] = 2;		// 2  = A2 - Vout
	ChSel[2] = 4;		// 4  = A4 - Ipri
		
	ChSel[8] = 9;		// 9  = B1 - LED string 1
	ChSel[9] = 10;		// 10 = B2 - LED string 2
	ChSel[10] = 11;		// 11 = B3 - LED string 3
	ChSel[11] = 12;		// 12 = B4 - LED string 4
	ChSel[12] = 14;		// 14 = B6 - LED string 5
	ChSel[13] = 3;		// 15 = B7 - LED string 6
	ChSel[14] = 1; 		// 1  = A1 - Vpfcout
	

	// ADC Trigger Selection
	TrigSel[0] = ADCTRIG_EPWM4_SOCA;	// ePWM4, ADCSOCA
	TrigSel[1] = ADCTRIG_EPWM4_SOCA;	// ePWM4, ADCSOCA
	TrigSel[2] = ADCTRIG_EPWM4_SOCA;	// ePWM4, ADCSOCA
	
	TrigSel[8] = ADCTRIG_EPWM1_SOCA;	// ePWM1, ADCSOCA
	TrigSel[9] = ADCTRIG_EPWM1_SOCB;	// ePWM1, ADCSOCB
	TrigSel[10] = ADCTRIG_EPWM2_SOCA;	// ePWM2, ADCSOCA
	TrigSel[11] = ADCTRIG_EPWM2_SOCB;	// ePWM2, ADCSOCB
	TrigSel[12] = ADCTRIG_EPWM3_SOCA;	// ePWM3, ADCSOCA
	TrigSel[13] = ADCTRIG_EPWM3_SOCB;	// ePWM3, ADCSOCB
	TrigSel[14] = ADCTRIG_EPWM3_SOCB;	// ePWM3, ADCSOCB
	

   	// Configure ADC 
	ADC_SOC_CNF(ChSel, TrigSel, ACQPS, 17, 0); 
	
	// Configure ePWMs to generate ADC SOC pulses
	EPwm4Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM4 SOCA pulse
	EPwm4Regs.ETSEL.bit.SOCASEL = ET_CTR_ZERO;		// SOCA from ePWM4 Zero event
	EPwm4Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM4 SOCA on every event

	EPwm1Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM1 SOCA pulse
	EPwm1Regs.ETSEL.bit.SOCASEL = ET_CTR_PRD;		// SOCA from ePWM1 PRD event
	EPwm1Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM1 SOCA on every event
	
	EPwm1Regs.ETSEL.bit.SOCBEN = 1;					// Enable ePWM1 SOCB pulse
	EPwm1Regs.ETSEL.bit.SOCBSEL = ET_CTR_ZERO;		// SOCB from ePWM1 Zero event
	EPwm1Regs.ETPS.bit.SOCBPRD = ET_1ST;			// Trigger ePWM1 SOCB on every event

	EPwm2Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM1 SOCA pulse
	EPwm2Regs.ETSEL.bit.SOCASEL = ET_CTR_PRD;		// SOCA from ePWM1 PRD event
	EPwm2Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM1 SOCA on every event
	
	EPwm2Regs.ETSEL.bit.SOCBEN = 1;					// Enable ePWM1 SOCB pulse
	EPwm2Regs.ETSEL.bit.SOCBSEL = ET_CTR_ZERO;		// SOCB from ePWM1 Zero event
	EPwm2Regs.ETPS.bit.SOCBPRD = ET_1ST;			// Trigger ePWM1 SOCB on every event
	
	EPwm3Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM1 SOCA pulse
	EPwm3Regs.ETSEL.bit.SOCASEL = ET_CTR_PRD;		// SOCA from ePWM1 PRD event
	EPwm3Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM1 SOCA on every event
	
	EPwm3Regs.ETSEL.bit.SOCBEN = 1;					// Enable ePWM1 SOCB pulse
	EPwm3Regs.ETSEL.bit.SOCBSEL = ET_CTR_ZERO;		// SOCB from ePWM1 Zero event
	EPwm3Regs.ETPS.bit.SOCBPRD = ET_1ST;			// Trigger ePWM1 SOCB on every event	


	EALLOW;
	// On RevA silcon, the ADC mode can be changed to non-overlap mode in order to 
	// avoid the ADC first sample issue (see device errata). Instead of double-sampling
	// the first sample as is done in this software, the following line can be commented
	// out to fix any issue in RevA silicon.
	// **** This is not done here because customer may use Rev0 silicon on this board.
	//AdcRegs.ADCCTL2.bit.ADCNONOVERLAP = 1;
	
	// Remove hysteresis from ADC channel A2 to improve voltage sensing
	AdcRegs.COMPHYSTCTL.bit.COMP1_HYST_DISABLE = 0x1;
	EDIS;

//=================================================================================
//	Configure COMPARATORS/TRIP ZONE Circuitry for System Level Protection
//=================================================================================
	EALLOW;
	
	//Overvoltage protection
	Comp1Regs.COMPCTL.bit.SYNCSEL = 0;		// asynchronous
	Comp1Regs.COMPCTL.bit.CMPINV = 1;		// invert output
	Comp1Regs.COMPCTL.bit.COMPSOURCE = 0;	// use internal DAC
	Comp1Regs.COMPCTL.bit.COMPDACEN = 1;	// enable comparator
	Comp1Regs.DACVAL.bit.DACVAL = 990;  	//number is in Q5    (990 * 3.3 / 2^10 = 3.19V =>50V)

	//Overcurrent protection
	Comp2Regs.COMPCTL.bit.SYNCSEL = 0;		// asynchronous
	Comp2Regs.COMPCTL.bit.CMPINV = 1;		// invert output
	Comp2Regs.COMPCTL.bit.COMPSOURCE = 0;	// use internal DAC
	Comp2Regs.COMPCTL.bit.COMPDACEN = 1;	// enable comparator
	Comp2Regs.DACVAL.bit.DACVAL = 990;  	// number is in Q5    (990 * 3.3 / 2^10 = 3.18V =>48V)

	//Enable Comparators to trip PWM4
	EPwm4Regs.DCTRIPSEL.bit.DCAHCOMPSEL = DC_COMP1OUT;   // DCAH = Comparator 1 output
    EPwm4Regs.DCTRIPSEL.bit.DCBHCOMPSEL = DC_COMP2OUT;        // DCAL = TZ2
    EPwm4Regs.TZDCSEL.bit.DCAEVT1 = TZ_DCAH_LOW; // DCAEVT1 =  DCAH low(will become active as Comparator1 output goes low)
    EPwm4Regs.TZDCSEL.bit.DCBEVT1 = TZ_DCBH_LOW; // DCAEVT1 =  DCAH low(will become active as Comparator2 output goes low) 
    EPwm4Regs.DCACTL.bit.EVT1SRCSEL = DC_EVT1;           // DCAEVT1 = DCAEVT1 (not filtered)
	EPwm4Regs.DCBCTL.bit.EVT1SRCSEL = DC_EVT1;           // DCAEVT1 = DCAEVT1 (not filtered)
    EPwm4Regs.DCACTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;  // Take async path
	EPwm4Regs.DCBCTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;  // Take async path
         
    EPwm4Regs.TZSEL.bit.DCAEVT1 = 1;
    EPwm4Regs.TZSEL.bit.DCBEVT1 = 1;
	
    // What do we want the DCAEVT1 and DCBEVT1 events to do? - DCAEVTx events can force EPWMxA  
    EPwm4Regs.TZCTL.bit.TZA = TZ_FORCE_LO;           // EPWM4A will go low
	EDIS;


//=================================================================================
//	Initialize Digital Power Library
//=================================================================================

	DPL_Init();


//==================================================================================
//	INCREMENTAL BUILD OPTIONS - NOTE: selected via {IsoACLighting-Settings.h
//==================================================================================
// ---------------------------------- USER -----------------------------------------	
//----------------------------------------------------------------------
#if (INCR_BUILD == 1) 	// Open Loop
//----------------------------------------------------------------------

	// Lib Module connection to "nets" 
	//----------------------------------------
	// Connect the PWM Driver input to an input variable, Open Loop System
	ADCDRV_1ch_Rlt1 = &Vout_LLC;
	ADCDRV_1ch_Rlt2 = &Ipri_LLC;
	ADCDRV_1ch_Rlt8 = &Iled[0];
	ADCDRV_1ch_Rlt9 = &Iled[1];
	ADCDRV_1ch_Rlt10 = &Iled[2];
	ADCDRV_1ch_Rlt11 = &Iled[3];
	ADCDRV_1ch_Rlt12 = &Iled[4];
	ADCDRV_1ch_Rlt13 = &Iled[5];
	
	PWMDRV_LLC_ComplPairDB_Duty4 = &llc_duty;
	PWMDRV_LLC_ComplPairDB_Period4 = &llc_period;
	
	PWMDRV_DualUpDwnCnt_Duty1A = &led_duty[0];
	PWMDRV_DualUpDwnCnt_Duty1B = &led_duty[1];
	PWMDRV_DualUpDwnCnt_Duty2A = &led_duty[2];
	PWMDRV_DualUpDwnCnt_Duty2B = &led_duty[3];
	PWMDRV_DualUpDwnCnt_Duty3A = &led_duty[4];
	PWMDRV_DualUpDwnCnt_Duty3B = &led_duty[5];
	
#endif // (INCR_BUILD == 1)

//----------------------------------------------------------------------
#if (INCR_BUILD == 2) 	// Closed Loop
//----------------------------------------------------------------------

	SlewStep_LLC = 2000;
	SlewStep_LED = 2000;

	// Lib Module connection to "nets" 
	//----------------------------------------
	// Connect the PWM Driver input to an adc variable, Closed Loop System
	ADCDRV_1ch_Rlt1 = &Vout_LLC;
	ADCDRV_1ch_Rlt2 = &Ipri_LLC;
	ADCDRV_1ch_Rlt8 = &Iled[0];
	ADCDRV_1ch_Rlt9 = &Iled[1];
	ADCDRV_1ch_Rlt10 = &Iled[2];
	ADCDRV_1ch_Rlt11 = &Iled[3];
	ADCDRV_1ch_Rlt12 = &Iled[4];
	ADCDRV_1ch_Rlt13 = &Iled[5];
	
	CNTL_2P2Z_Ref1 = &Vset_LLC_slewed;
	CNTL_2P2Z_Fdbk1 = &Vout_LLC;
	CNTL_2P2Z_Out1 = &llc_period;
	CNTL_2P2Z_Coef1 = &CNTL_2P2Z_CoefStruct_LLC.b2;
	
	CNTL_2P2Z_Ref2 = &Iset_LED_slewed[0];
	CNTL_2P2Z_Fdbk2 = &Iled[0];
	CNTL_2P2Z_Out2 = &led_duty[0];
	CNTL_2P2Z_Coef2 = &CNTL_2P2Z_CoefStruct_LED.b2;
	
	CNTL_2P2Z_Ref3 = &Iset_LED_slewed[1];
	CNTL_2P2Z_Fdbk3 = &Iled[1];
	CNTL_2P2Z_Out3 = &led_duty[1];
	CNTL_2P2Z_Coef3 = &CNTL_2P2Z_CoefStruct_LED.b2;
	
	CNTL_2P2Z_Ref4 = &Iset_LED_slewed[2];
	CNTL_2P2Z_Fdbk4 = &Iled[2];
	CNTL_2P2Z_Out4 = &led_duty[2];
	CNTL_2P2Z_Coef4 = &CNTL_2P2Z_CoefStruct_LED.b2;
	
	CNTL_2P2Z_Ref5 = &Iset_LED_slewed[3];
	CNTL_2P2Z_Fdbk5 = &Iled[3];
	CNTL_2P2Z_Out5 = &led_duty[3];
	CNTL_2P2Z_Coef5 = &CNTL_2P2Z_CoefStruct_LED.b2;
	
	CNTL_2P2Z_Ref6 = &Iset_LED_slewed[4];
	CNTL_2P2Z_Fdbk6 = &Iled[4];
	CNTL_2P2Z_Out6 = &led_duty[4];
	CNTL_2P2Z_Coef6 = &CNTL_2P2Z_CoefStruct_LED.b2;
	
	CNTL_2P2Z_Ref7 = &Iset_LED_slewed[5];
	CNTL_2P2Z_Fdbk7 = &Iled[5];
	CNTL_2P2Z_Out7 = &led_duty[5];
	CNTL_2P2Z_Coef7 = &CNTL_2P2Z_CoefStruct_LED.b2;

	PWMDRV_LLC_ComplPairDB_Duty4 = &llc_duty;
	PWMDRV_LLC_ComplPairDB_Period4 = &llc_period;
	
	PWMDRV_DualUpDwnCnt_Duty1A = &led_duty[0];
	PWMDRV_DualUpDwnCnt_Duty1B = &led_duty[1];
	PWMDRV_DualUpDwnCnt_Duty2A = &led_duty[2];
	PWMDRV_DualUpDwnCnt_Duty2B = &led_duty[3];
	PWMDRV_DualUpDwnCnt_Duty3A = &led_duty[4];
	PWMDRV_DualUpDwnCnt_Duty3B = &led_duty[5];
	
#endif // (INCR_BUILD == 2)

//====================================================================================
// INTERRUPTS & ISR INITIALIZATION (best to run this section after other initialization)
//====================================================================================

	Comms_Init();

//Also Set the appropriate # define's in the HVLLC-Settings.h 
//to enable interrupt management in the ISR
	EALLOW;
// Set up C28x Interrupt
// PWM based ISR trigger (Lighting + Resonant controller)
    PieVectTable.EPWM1_INT = &DPL_ISR;      		// Map Interrupt
   	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;      		// PIE level enable, Grp3 / Int1, ePWM1
   	EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_PRDZERO; 	// INT on ComparePRD and ZRO event
   	EPwm1Regs.ETSEL.bit.INTEN = 1;              	// Enable INT
    EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;         	// Generate INT on every 1st event
	
// PWM based ISR trigger (Resonant PWM)
	PieVectTable.EPWM4_INT = &RESPWM_ISR;		// Map Interrupt
	PieCtrlRegs.PIEIER3.bit.INTx4 = 1;			// PIE level enable, Grp3 / Int4
	EPwm4Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;	// INT on Counter-Zero event
	EPwm4Regs.ETSEL.bit.INTEN = 1;
	EPwm4Regs.ETPS.bit.INTPRD = ET_1ST;
    
    IER |= M_INT3;                          	// Enable CPU INT3 connected to EPWM1-6 INTs:
    
    EINT;                                   	// Enable Global interrupt INTM
    ERTM;                                   	// Enable Global realtime interrupt DBGM     

//=================================================================================
//	BACKGROUND (BG) LOOP
//=================================================================================

//--------------------------------- FRAMEWORK -------------------------------------
	for(;;)  //infinite loop
	{
		// State machine entry & exit point
		//===========================================================
		(*Alpha_State_Ptr)();	// jump to an Alpha state (A0,B0,...)
		//===========================================================

	}
} //END MAIN CODE



//=================================================================================
//	STATE-MACHINE SEQUENCING AND SYNCRONIZATION
//=================================================================================

//--------------------------------- FRAMEWORK -------------------------------------
void A0(void)
{
	// loop rate synchronizer for A-tasks
	if(CpuTimer0Regs.TCR.bit.TIF == 1)
	{
		CpuTimer0Regs.TCR.bit.TIF = 1;	// clear flag
		//-----------------------------------------------------------
		(*A_Task_Ptr)();		// jump to an A Task (A1,A2,A3,...)
		//-----------------------------------------------------------
		VTimer0[0]++;			// virtual timer 0, instance 0 (spare)
	}
	Alpha_State_Ptr = &B0;		// Comment out to allow only A tasks
}

void B0(void)
{
	// loop rate synchronizer for B-tasks
	if(CpuTimer1Regs.TCR.bit.TIF == 1)
	{
		CpuTimer1Regs.TCR.bit.TIF = 1;				// clear flag
		//-----------------------------------------------------------
		(*B_Task_Ptr)();		// jump to a B Task (B1,B2,B3,...)
		//-----------------------------------------------------------
		VTimer1[0]++;			// virtual timer 1, instance 0 (spare)
	}
	Alpha_State_Ptr = &C0;		// Allow C state tasks
}

void C0(void)
{
	// loop rate synchronizer for C-tasks
	if(CpuTimer2Regs.TCR.bit.TIF == 1)
	{
		CpuTimer2Regs.TCR.bit.TIF = 1;				// clear flag
		//-----------------------------------------------------------
		(*C_Task_Ptr)();		// jump to a C Task (C1,C2,C3,...)
		//-----------------------------------------------------------
		VTimer2[0]++;			//virtual timer 2, instance 0 (spare)
	}
	Alpha_State_Ptr = &A0;	// Back to State A0
}


//=================================================================================
//	A - TASKS
//=================================================================================
//--------------------------------------------------------
void A1(void) 	//Enable/Disable LLC converter; Slewed ramping of LLC output voltage; comms
//--------------------------------------------------------
{	
	HighRef = 6000;
	
	if(ResonantEnable == 1)
	{
		#if(INCR_BUILD == 2)
		CNTL_2P2Z_Fdbk1 = &Vout_LLC;
		#endif
		(*ePWM[4]).AQCSFRC.bit.CSFA = 0;
	}
	else
	{
		#if(INCR_BUILD == 2)
		Gui_Vset_LLC = 0;					//reset LLC reference to 0 to avoid issues when turning LLC back on
		CNTL_2P2Z_Fdbk1 = &HighRef;
		#endif
		(*ePWM[4]).AQCSFRC.bit.CSFA = 1;
	}
	
	Comms_Run();

	//---------------------------------------------------------
	SlewError = (Vset_LLC_slewed - Vset_LLC);
	if (SlewError > SlewStep_LLC)
	{		
		Vset_LLC_slewed = Vset_LLC_slewed - SlewStep_LLC;
	}
	else if (SlewError < (-SlewStep_LLC) )
	{
		Vset_LLC_slewed = Vset_LLC_slewed + SlewStep_LLC;
	}
	else	Vset_LLC_slewed = Vset_LLC;
	//---------------------------------------------------------


	//-------------------
	//the next time CpuTimer0 'counter' reaches Period value go to A2
	A_Task_Ptr = &A2;
	//-------------------
}

//-----------------------------------------------------------------
void A2(void)   // Slewed ramping of LED string currents; LED_linked functionality
//-----------------------------------------------------------------
{	
	for(i=0;i<6;i++)
	{
		//---------------------------------------------------------
		SlewError = (Iset_LED_slewed[i] - Iset_LED[i]);
		if (SlewError > SlewStep_LED)			
		{
			if(Iset_LED_slewed[i] < LedMinCurrent)	Iset_LED_slewed[i] = 0;
			else Iset_LED_slewed[i] = Iset_LED_slewed[i] - SlewStep_LED;
		}
		else if (SlewError < (-SlewStep_LED) )
		{			
			if(Iset_LED_slewed[i] < LedMinCurrent)	Iset_LED_slewed[i] = LedMinCurrent;
			else Iset_LED_slewed[i] = Iset_LED_slewed[i] + SlewStep_LED;
		}
		else		Iset_LED_slewed[i] = Iset_LED[i];
	}

	if (LEDs_linked == 1)
	{
		for (i=0; i<6; i++)
		{
			Gui_IsetLed[i] = Gui_IsetLed[0];
		}
	}
	
	//-------------------
	//the next time CpuTimer0 'counter' reaches Period value go to A1
	A_Task_Ptr = &A1;
	//-------------------
}


//=================================================================================
//	B - TASKS
//=================================================================================

//----------------------------------- USER ----------------------------------------

//----------------------------------------
void B1(void) 		// Force LED output low if Iref is 0 and Update Coefficients	 
//----------------------------------------
{	
	#if (INCR_BUILD == 2)
	Zero = 0;
	HighRef = 6000;					//fool 2P2Z into clearing its internal history
	
	if(Iset_LED_slewed[0] == 0)
	{
		PWMDRV_DualUpDwnCnt_Duty1A = &Zero;
		CNTL_2P2Z_Fdbk2 = &HighRef;
		led_duty[0] = 0;
	}
	else
	{
		CNTL_2P2Z_Fdbk2 = &Iled[0];
		PWMDRV_DualUpDwnCnt_Duty1A = &led_duty[0];
	}
	
	if(Iset_LED_slewed[1] == 0)
	{
		PWMDRV_DualUpDwnCnt_Duty1B = &Zero;
		CNTL_2P2Z_Fdbk3 = &HighRef;
		led_duty[1] = 0;
	}
	else
	{
		CNTL_2P2Z_Fdbk3 = &Iled[1];
		PWMDRV_DualUpDwnCnt_Duty1B = &led_duty[1];
	}
	
	if(Iset_LED_slewed[2] == 0)
	{
		PWMDRV_DualUpDwnCnt_Duty2A = &Zero;
		CNTL_2P2Z_Fdbk4 = &HighRef;
		led_duty[2] = 0;
	}
	else
	{
		CNTL_2P2Z_Fdbk4 = &Iled[2];
		PWMDRV_DualUpDwnCnt_Duty2A = &led_duty[2];
	}
	
	if(Iset_LED_slewed[3] == 0)
	{
		PWMDRV_DualUpDwnCnt_Duty2B = &Zero;
		CNTL_2P2Z_Fdbk5 = &HighRef;
		led_duty[3] = 0;
	}
	else
	{
		CNTL_2P2Z_Fdbk5 = &Iled[3];
		PWMDRV_DualUpDwnCnt_Duty2B = &led_duty[3];
	}
	
	if(Iset_LED_slewed[4] == 0)
	{
		PWMDRV_DualUpDwnCnt_Duty3A = &Zero;
		CNTL_2P2Z_Fdbk6 = &HighRef;
		led_duty[4] = 0;
	}
	else
	{
		CNTL_2P2Z_Fdbk6 = &Iled[4];
		PWMDRV_DualUpDwnCnt_Duty3A = &led_duty[4];
	}
	
	if(Iset_LED_slewed[5] == 0)
	{
		PWMDRV_DualUpDwnCnt_Duty3B = &Zero;
		CNTL_2P2Z_Fdbk7 = &HighRef;
		led_duty[5] = 0;
	}
	else
	{
		CNTL_2P2Z_Fdbk7 = &Iled[5];
		PWMDRV_DualUpDwnCnt_Duty3B = &led_duty[5];
	}
	
	CNTL_2P2Z_CoefStruct_LLC.b2 = LLC_CONTROL_POLARITY*(DgainLLC);
	CNTL_2P2Z_CoefStruct_LLC.b1 = LLC_CONTROL_POLARITY*(IgainLLC - PgainLLC - DgainLLC - DgainLLC);
	CNTL_2P2Z_CoefStruct_LLC.b0 = LLC_CONTROL_POLARITY*(PgainLLC + IgainLLC + DgainLLC);

	CNTL_2P2Z_CoefStruct_LED.b2 = DgainLED;
	CNTL_2P2Z_CoefStruct_LED.b1 = IgainLED - PgainLED - DgainLED - DgainLED;
	CNTL_2P2Z_CoefStruct_LED.b0 = PgainLED + IgainLED + DgainLED;

	#endif

	//-----------------
	//the next time CpuTimer1 'counter' reaches Period value go to B1
	B_Task_Ptr = &B1;	
	//-----------------
}

//=================================================================================
//	C - TASKS
//=================================================================================

//--------------------------------- USER ------------------------------------------

//------------------------------------------------------
void C1(void)  //Convert LLC output voltage and current to "real-world" values
//------------------------------------------------------
{
	HistPtr++;
	if (HistPtr >= HistorySize)	
	{
		HistPtr = 0;
		
		// C Tasks (based on CpuTimer2) run at 10ms * 2 tasks interlaced * 8 (HistPtr range) = 160ms toggle speed
		GpioDataRegs.GPATOGGLE.bit.GPIO12 = 1;
	}

	// BoxCar Averages - Input Raw samples into BoxCar arrays
	//----------------------------------------------------------------
	Hist_Vout_LLC[HistPtr] = VoutR;
	Hist_Ipri_LLC[HistPtr] = IpriR;

	// Measurements
	//----------------------------------------------------------------
	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Vout_LLC[i];	//Q12 * 8 = Q15
	Gui_Vout_LLC = ((long)temp_Scratch*(long)K_Vout_LLC) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Ipri_LLC[i];	//Q12 * 8 = Q15
	Gui_Ipri_LLC = ((long)temp_Scratch*(long)K_Ipri_LLC) >> 15;	//Q15*Q15 >> 15 = Q15

	//Multiply with longs to get proper result then shift by 5 to turn it back into Q24
	Vset_LLC = ((long)Gui_Vset_LLC*(long)iK_Vset_LLC) >> 5;
	

	//-----------------
	//the next time CpuTimer2 'counter' reaches Period value go to C2
	C_Task_Ptr = &C2;	
	//-----------------
}	

//----------------------------------------
void C2(void) // Convert LED string currents to "real world" values
//----------------------------------------
{
	// LED String Measurements
	//----------------------------------------------------------------
	// view following variables in Watch Window as:
	//		Gui_ILed[j] = Q15

	ILedH[0][HistPtr]   = LED1R;
	ILedH[1][HistPtr]   = LED2R;
	ILedH[2][HistPtr]	= LED3R;
	ILedH[3][HistPtr]	= LED4R;
	ILedH[4][HistPtr]   = LED5R;
	ILedH[5][HistPtr]   = LED6R;

	for(j = 0; j<6; j++)
	{
		temp_Scratch=0;
		for(i=0; i<HistorySize; i++)	temp_Scratch = temp_Scratch + ILedH[j][i];
		Gui_ILed[j] = ( (long) temp_Scratch * (long) K_ILed ) >> 15;

		Iset_LED[j] = ( (long) Gui_IsetLed[j] * (long) iK_ILed ) >> 5;  //Iset_LED is Q24
	}
	
			
	//-----------------
	//the next time CpuTimer1 'counter' reaches Period value go to B1
	C_Task_Ptr = &C1;
	//-----------------
}
